#include <vector>
#include <iostream>
#include <string>


template<class T>
struct comparer{
    bool operator()(const T& a, const T& b){
        return a>b;
    }
};

template<>
struct comparer<std::string>{
    bool operator()(const std::string& a, const std::string& b){
        for (int i=0; i < std::min(a.size(),b.size());i++){
            if (a[i] > b[i]) return true;
            if (a[i] < b[i]) return false;
        }
        return a.size() > b.size();
    }
};

template<>
struct comparer<std::pair<std::string,int> >{
    bool operator()(const std::pair<std::string,int> & a, const std::pair<std::string,int>& b){
        return a.second > b.second;
    }
};

template<class T, class greater=comparer<T> >
void Tri (std::vector<T>& v){
    for (int i=1; i<v.size(); i++)
        for (int j=0; j<i; j++)
            if (greater()(v[j],v[i]))
                std::swap(v[i],v[j]);
}

template<class T>
std::ostream& operator<<(std::ostream& o, std::vector<T>& v){
   for (auto x:v)
        o << x <<" ";
   return o;
}
template<class T,class U>
std::ostream& operator<<(std::ostream& o, std::pair<T,U>& v){
    o << "(" << v.first << ", "<< v.second <<") ";
   return o;
}

template<class T>
struct CompareStringLength{
    bool operator()(const T& a, const T& b){
        return a.size() > b.size();
    }
};

int main(){
    std::vector<int> v={3,9,4,1,7,5};
    Tri(v);
    
    std::cout << v <<std::endl;

    std::vector<std::string> w = {"bonjour","coucou","insertion","bla"};
    Tri (w); // comparaison lexicographique 
    std::cout<< w <<std::endl;

    Tri<std::string, CompareStringLength<std::string> > (w); // comparaison des longueurs
    std::cout<< w <<std::endl;

    std::vector< std::pair<std::string,int> > z = {{"bonjour",2},{"coucou",1515},{"gazobu",1},{"L3MI",7}};

    Tri (z);
    std::cout<< z <<std::endl;
}
